// **********************************************************************
//
// Copyright (c) 1999
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1999
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_OBJECT_H
#define OB_OBJECT_H

namespace CORBA
{
    
class Object;
typedef Object* Object_ptr;
typedef Object_ptr ObjectRef;

} // End of namespace CORBA

void OBDuplicate(CORBA::Object_ptr);
void OBRelease(CORBA::Object_ptr);

namespace CORBA
{

//
// The SetOverrideType
//
enum SetOverrideType
{
    SET_OVERRIDE,
    ADD_OVERRIDE
};

//
// The Object class
//
typedef OB::ObjVar< Object > Object_var;
typedef OB::ObjOut< Object > Object_out;

class Object : public OB::RefCount
{
    //
    // Hide copy-constructor and assignment operator
    //
    Object(const Object&);
    void operator=(const Object&);

    static const char* _ob_ids_[];

protected:

    //
    // The IOR
    //
    IOP::IOR _ob_ior_;

    //
    // The set of policies
    //
    PolicyList _ob_policies_;

    //
    // Locality constrained?
    //
    bool _ob_locality_;
    
#ifdef HAVE_JTC
    //
    // This mutex protects the _ob_stubImpl_ member
    //
    JTCRecursiveMutex _ob_implMutex_;

    //
    // This mutex protects the _ob_ior_ member
    //
    JTCMutex _ob_iorMutex_;
#endif

    //
    // The stub implementation.
    //
    OB::StubImplBase_var _ob_stubImpl_;

    OB::StubImplBase_ptr _OB_getStubImpl();
    virtual void _OB_clearStubImpl();

    Object();
    friend class ORB; // The ORB may create Objects

public:

    virtual ~Object();

    //
    // Standard IDL to C++ Mapping
    //
    friend void release(Object_ptr);
    friend Boolean is_nil(Object_ptr);
    static inline Object_ptr _duplicate(Object_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline Object_ptr _nil()
    { return 0; }

    virtual InterfaceDef_ptr _get_interface();

    virtual Boolean _is_a(const char*);
    virtual Boolean _non_existent();
    Boolean _is_equivalent(Object_ptr);
    ULong _hash(ULong);

    void _create_request(Context_ptr, const char*,
                         NVList_ptr, NamedValue_ptr,
                         Request_out, Flags);
    
    void _create_request(Context_ptr, const char*,
                         NVList_ptr, NamedValue_ptr,
                         ExceptionList_ptr,
                         ContextList_ptr,
                         Request_out, Flags);

    Request_ptr _request(const char* operation);

    Policy_ptr _get_policy(PolicyType);
    Object_ptr _set_policy_overrides(const PolicyList&, SetOverrideType);
    
    //XXX
    //DomainManagerList* _get_domain_managers();

    //
    // Policies
    // From draft messaging specifiction
    //
    Policy_ptr _get_effective_policy_override(PolicyType);
    PolicyList* _get_policy_overrides(const PolicyTypeSeq&);
    Boolean _validate_policies(const PolicyList&);

    //
    // Additonal ORBacus specific functions
    //
    OCI::TransportInfo_ptr _get_oci_transport_info();
    OCI::ConnectorInfo_ptr _get_oci_connector_info();

    //
    // Return a null-terminated list of the object's repository IDs
    //
    virtual const char** _OB_ids() const;
    static const char** _OB_staticIds();

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    const IOP::IOR& _OB_ior() { return _ob_ior_; }
    void _OB_ior(const IOP::IOR& ior) { _ob_ior_ = ior; }

    const PolicyList& _OB_policies();
    void _OB_policies(const PolicyList&);
    Policy_ptr _OB_getPolicy(PolicyType policyType);
    
    virtual const char* _OB_typeId() const;

    void _OB_copyFrom(Object_ptr);

    void _OB_forward(const IOP::IOR&);

    static void _OB_unmarshal(Object_ptr&, OB::InputStream&);

    virtual OB::StubImplBase* _OB_createStubImpl();

    OB::StubImplBase* _OB_getImplBase();

    Boolean _OB_isInternalLocal();

    virtual AbstractBase_ptr _OB_toAbstractBase();

    friend class OB::InputStream;
    friend class OB::OutputStream;
    friend class OB::CountStream;
};

//
// release and is_nil
//
inline void
release(Object_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(Object_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

namespace OB
{

//
// StubImpl class for CORBA::Object
//
class ObjectStubImpl : public StubImplBase
{
    //
    // Hide copy-constructor and assignment operator
    //
    ObjectStubImpl(const ObjectStubImpl&);
    void operator=(const ObjectStubImpl&);

public:              

    ObjectStubImpl() { }
};

} // End of namespace OB

#endif
